#ifndef __MD_INTEGRATOR__
#define __MD_INTEGRATOR__

#include <Stringpp.h>
#include <MD_BC.h>
#include <Cell.h>
#include <MD_Potential.h>
#include <Calcul_timer.h>
#include <Particle.h>

namespace ZSET {
  using namespace ZSET;

class MD_INTEGRATOR 
{
protected:
  STRING name;
  bool periodic_x, periodic_y, periodic_z,	
       rigid_x_right, rigid_x_left, 
       rigid_y_top, rigid_y_bot, 
       rigid_z_top, rigid_z_bot,
       has_rigid_bc;  
  VECTOR rmin,rmax;
public:
  MD_INTEGRATOR(STRING n) {if (n == SV) name = n; else ERROR("Unknown integrator"); }
  MD_INTEGRATOR() {}
 ~MD_INTEGRATOR() {}

  int escaping_node(VECTOR e);
  void init(ARRAY<CELL*>& c);
  void update_region_bc(ARRAY<MD_BC*> bc);
  void compX(ARRAY<CELL*>& cells, double dt);
  void redistribute_particles(ARRAY<CELL*>& cells);
  //void updateX(PARTICLE*& p, double dt);
  void compV(ARRAY<CELL*>& cells, double dt);
  void compV(ARRAY<CELL*>& cells, double dt, double& kinetic_energy);
  //void updateV(PARTICLE*& p, double dt);
  void compF(ARRAY<CELL*>& cells, MD_POTENTIAL* pot);
  void compF(ARRAY<CELL*>& cells, MD_POTENTIAL* pot, double& potential_energy, double& pressure);
  int  solve_step(ARRAY<CELL*>& cells, MD_POTENTIAL* pot, double dt);
  int  solve_step(ARRAY<CELL*>& cells, MD_POTENTIAL* pot, double dt, bool compute_energy, double& kinetic_energy, double& potential_energy, double& pressure, CALC_TIMER_COUNTER& tc);
};

}

#endif

